module RipeDbClient

  #
  # Error Class - Inherits from Standard Error
  #
  # @author Mike Simkins
  #
  class Error < StandardError
  end


  # This error is raised when a user has not specified a value for a mandatory parameter
  #
  #   raise RipeDbClient::BadParameter.new("search parameter missing", "dblookup")
  #
  # The passed message, action, and subject are optional and can later be retrieved when
  # rescuing from the exception.
  #
  #   exception.message # => "search parameter missing"
  #   exception.action # => "dblookup"
  #
  # If the message is not specified (or is nil) it will default to "You have an Invalid or Missing Parameter"
  # This default can be overridden by setting default_message.
  #
  #   exception.default_message = "Default error message"
  #   exception.message # => "Default error message"
  #
  #
  # @author Mike Simkins
  #
  class BadParameter < Error
    attr_reader :action, :subject
    attr_writer :default_message

    def initialize (message = nil, action = nil, subject = nil)
      @message = message
      @action = action
      @subject = subject
      @default_message = "You have an Invalid or Missing Parameter"
    end


    # @return [String] Returns the error message
    def to_s
      @message || @default_message
    end
  end

  # This error is raised when a user tries to access an unsupported function
  #
  #   raise RipeDbClient::Unsupported.new("my secret feature", "top secret")
  #
  # The passed message, action, and subject are optional and can later be retrieved when
  # rescuing from the exception.
  #
  #   exception.message # => "my secret feature"
  #   exception.action # => "top_secret"
  #
  # If the message is not specified (or is nil) it will default to "The feature you require is not currently supported"
  # This default can be overridden by setting default_message.
  #
  #   exception.default_message = "Default error message"
  #   exception.message # => "Default error message"
  #
  # @author Mike Simkins
  #
  class Unsupported < Error
    attr_reader :action, :subject
    attr_writer :default_message

    def initialize (message = nil, action = nil, subject = nil)
      @message = message
      @action = action
      @subject = subject
      @default_message = "The feature you require is not currently supported"
    end

    # @return [String] Returns the error message
    def to_s
      @message || @default_message
    end
  end
end